@@ -42,7 +42,11 @@ def get_coords_arr(images, patch_size, step_size):
     grid_list = []
     image_shape = torch.tensor([images.shape[-3], images.shape[-2], images.shape[-1]])
     for i in range(3):
-        grid_list.append(torch.arange(1+(image_shape[i] - patch_size[i])/(patch_size[i] * step_size[i])))
+        step = 1+(image_shape[i] - patch_size[i])/(patch_size[i] * step_size[i])
+        if step < 1:
+            print('step is zero')
+            step = 1
+        grid_list.append(torch.arange(step))
     grid_x, grid_y, grid_z = torch.meshgrid(*grid_list)
     grid_x = torch.round(grid_x * step_size[0] * patch_size[0]).long()
     grid_y = torch.round(grid_y * step_size[1] * patch_size[1]).long()
@@ -74,7 +78,6 @@ def get_patch_from_coords(image, patch_coords, index):
     return result
 
 def change_value_from_coords(image, patch_coords, index, patch):
-    print(image.shape, patch.shape)
     for i in range(image.shape[0]):
         image[i][torch.meshgrid(*patch_coords[index])] += patch[i]
     return image
@@ -117,6 +120,7 @@ class InferenceV2(object):
         self.is_2d = is_2d
     
     def valid_run(self):
+        print('start valid run')
         final_score = []
         self.model = self.model.eval()
         for data_index in range(len(self.dataset)):
@@ -174,7 +178,7 @@ class InferenceV2(object):
             np.save(os.path.join(path, identifier + mode + '{0:03}.npy'.format(index)),image.detach().cpu().numpy())
         
         if is_nifti:
-            nii_img = sitk.GetImageFromArray(image)
+            nii_img = sitk.GetImageFromArray(image.detach().cpu().numpy())
             sitk.WriteImage(nii_img, os.path.join(self.nii_img_path, identifier + mode + '{0:03}.nii.gz'.format(index)))
 
     def test_run(self):
@@ -190,7 +194,9 @@ class InferenceV2(object):
         if self.padding:
             image = get_pad_images(image, self.patch_size)
         coords = get_coords_arr(image, self.patch_size, self.step_size)
+        self.model.eval()
 
+        print('get result')
         for coords_index in range(len(coords)):
             if coords_index == 0:
                 patch_bundle = get_patch_from_coords(image, coords, coords_index).unsqueeze(0)
@@ -207,8 +213,10 @@ class InferenceV2(object):
                 output = self.model(mini_batch_patch.to(self.device).float())
             if self.deep_supervision:
                 output = output[0]
-            output = output.detach().cpu()
+            output = torch.softmax(output, dim = 1)
+            output = output.cpu()
             for batch_index in range(mini_batch_index.shape[0]):
+                print('result shape', result.shape, 'coords x max', coords[mini_batch_index[batch_index]][0].max(), coords[mini_batch_index[batch_index]][1].max(), coords[mini_batch_index[batch_index]][2].max(), 'batch index', batch_index, 'mini batch index', mini_batch_index[batch_index])
                 result = change_value_from_coords(result, coords, mini_batch_index[batch_index], output[batch_index])
         
         if self.save_path != None:
@@ -218,10 +226,10 @@ class InferenceV2(object):
             result = torch.argmax(result, 0)
         
 
-            
         if self.save_path != None:
             self.save(result, self.model_path, data_index, current_name, mode = 'seg', is_nifti = True)
             self.save(image[0], self.image_path, data_index, current_name, mode = 'image', is_nifti = True)
+
         if self.postprocessing != None:
             classes = np.unique(result)
             new_result = torch.zeros(result.shape)
@@ -233,6 +241,8 @@ class InferenceV2(object):
                 new_result += class_map * i
             result = new_result
         
+        self.save(result, self.model_path, data_index, current_name, mode = 'postprocessing', is_nifti = True)
+        
         return result
 
 
