@@ -134,11 +134,12 @@ class Cropping(object):
 
 
 class Resampling(object):
-    def __init__(self, anisotropy_axis_index = None, target_spacing = [1.0, 1.0, 1.0], random = False, device = None):
+    def __init__(self, anisotropy_axis_index = None, target_spacing = [1.0, 1.0, 1.0], random = False, device = None, spacing_order = [-3, -2, -1]):
         self.device = device
         self.anisotropy_axis_index = self.new_anisotropy_axis(anisotropy_axis_index)
         self.target_spacing = np.array(target_spacing)
         self.random = random
+        self.spacing_order = spacing_order
     
     def new_anisotropy_axis(self, array):
         for i in range(len(array)):
@@ -158,7 +159,7 @@ class Resampling(object):
         original_spacing = info_data['spacing']
         if self.random:
             self.target_spacing = original_spacing * np.random.uniform(0.20, 5.0, size = np.array(original_spacing).shape)
-        new_shape = list((original_spacing/self.target_spacing * np.array([data.shape[-3], data.shape[-2], data.shape[-1]])).astype(int))
+        new_shape = list((original_spacing/self.target_spacing * np.array([data.shape[self.spacing_order[0]], data.shape[self.spacing_order[1]], data.shape[self.spacing_order[2]]])).astype(int))
         data = F.interpolate(data.unsqueeze(0), new_shape).squeeze(0)
 
         return data
@@ -232,11 +233,12 @@ class Get_target_spacing(object):
 
 
 class ResamplingV2(object):
-    def __init__(self, anisotropy_axis_index = [], target_spacing = [1.0, 1.0, 1.0], random = False, device = None):
+    def __init__(self, anisotropy_axis_index = [], target_spacing = [1.0, 1.0, 1.0], random = False, device = None, spacing_order = [-3, -2, -1]):
         self.device = device
         self.anisotropy_axis_index = self.new_anisotropy_axis(anisotropy_axis_index)
         self.target_spacing = np.array(target_spacing)
         self.random = random
+        self.spacing_order = spacing_order
     
     def new_anisotropy_axis(self, array):
         for i in range(len(array)):
@@ -265,7 +267,7 @@ class ResamplingV2(object):
         
         if self.random:
             self.target_spacing = original_spacing * np.random.uniform(0.20, 5.0, size = np.array(original_spacing).shape)
-        new_shape = list((original_spacing/self.target_spacing * np.array([img.shape[-3], img.shape[-2], img.shape[-1]])).astype(int))
+        new_shape = list((original_spacing/self.target_spacing * np.array([img.shape[self.spacing_order[0]], img.shape[self.spacing_order[1]], img.shape[self.spacing_order[2]]])).astype(int))
         
         img = F.interpolate(img.unsqueeze(0), new_shape, mode = 'trilinear').squeeze(0)
         
